class nick.yugopoMaze.Node
{
   var x;
   var leftX;
   var rightX;
   var y;
   var nodeType;
   var nodeClass;
   var roomWidth;
   var roomHeight;
   var roomAnchor;
   var doSpawn;
   var __roomProbability;
   var __deadEndProbability;
   var __INC_DEAD_PROBABILITY = 0.035;
   var __DEF_ROOM_PROBABILITY = 0.2;
   function Node(t_data)
   {
      this.x = this.leftX = this.rightX = t_data.x;
      this.y = t_data.y;
      this.nodeType = t_data.nodeType;
      this.nodeClass = t_data.nodeClass;
      this.roomWidth = this.roomHeight = this.roomAnchor = 0;
      if(t_data.doSpawn == undefined)
      {
         this.doSpawn = true;
      }
      else
      {
         this.doSpawn = t_data.doSpawn;
      }
      if(t_data.roomProbability == undefined)
      {
         this.__roomProbability = this.__DEF_ROOM_PROBABILITY;
      }
      else
      {
         this.__roomProbability = t_data.roomProbability;
      }
      if(t_data.deadProbability == undefined)
      {
         this.__deadEndProbability = 0;
      }
      else
      {
         this.__deadEndProbability = t_data.deadProbability;
      }
      if(this.nodeType == "room")
      {
         this.roomWidth = Math.floor(2 + Math.random() * 7);
         this.roomHeight = Math.round(1 + Math.random());
         if(Math.random() < 0.1)
         {
            this.roomHeight = 3;
         }
         this.roomAnchor = Math.floor(Math.random() * this.roomWidth);
         if(this.roomAnchor > this.roomWidth - 1)
         {
            this.roomAnchor = this.roomWidth - 1;
         }
         if(this.x + (this.roomWidth - 1 - this.roomAnchor) > nick.yugopoMaze.GameState.getInstance().MAP_WIDTH - 1)
         {
            this.roomAnchor += this.x + (this.roomWidth - 1 - this.roomAnchor) - (nick.yugopoMaze.GameState.getInstance().MAP_WIDTH - 1);
         }
         if(this.x - this.roomAnchor < 0)
         {
            this.roomAnchor += this.x - this.roomAnchor;
         }
         if(this.roomHeight > 1)
         {
            this.doSpawn = false;
         }
         this.leftX = this.x - this.roomAnchor;
         this.rightX = this.x + (this.roomWidth - 1 - this.roomAnchor);
      }
      else if(this.nodeType == "pillar")
      {
         this.roomWidth = t_data.roomWidth;
      }
   }
   function getNoFillSpaces()
   {
      if(this.nodeType == "room" || this.nodeType == "pillar")
      {
         return this.roomWidth;
      }
      return 1;
   }
   function generateSupportNode(isAllowedToEnd)
   {
      if(!this.doSpawn)
      {
         return undefined;
      }
      if(Math.random() < this.__deadEndProbability && isAllowedToEnd)
      {
         return undefined;
      }
      this.__deadEndProbability += this.__INC_DEAD_PROBABILITY;
      return new nick.yugopoMaze.Node({x:this.leftX,y:this.y - 1,nodeClass:"support",nodeType:this.getNextRowType(),roomProbability:this.__roomProbability,deadProbability:this.__deadEndProbability});
   }
   function generateCornerNodes()
   {
      var _loc2_ = [];
      _loc2_.push(new nick.yugopoMaze.Node({x:this.rightX,y:this.y,nodeClass:"support",nodeType:"corner",doSpawn:(this.roomHeight <= 1 ? true : false)}));
      if(this.roomHeight > 2)
      {
         _loc2_.push(new nick.yugopoMaze.Node({x:this.leftX,y:this.y - 1,nodeClass:"support",nodeType:"pillar",roomWidth:this.roomWidth,doSpawn:false}));
         _loc2_.push(new nick.yugopoMaze.Node({x:this.rightX,y:this.y - 1,nodeClass:"support",nodeType:"corner",doSpawn:false}));
      }
      if(this.roomHeight > 1)
      {
         _loc2_.push(new nick.yugopoMaze.Node({x:this.leftX,y:this.y - (this.roomHeight - 1),nodeClass:"support",nodeType:"pillar",roomWidth:this.roomWidth,doSpawn:(this.roomAnchor != 0 ? true : false),deadProbability:this.__deadEndProbability - this.__INC_DEAD_PROBABILITY}));
         _loc2_.push(new nick.yugopoMaze.Node({x:this.rightX,y:this.y - (this.roomHeight - 1),nodeClass:"support",nodeType:"corner"}));
      }
      return _loc2_;
   }
   function getNextRowType()
   {
      if(this.nodeClass == "root")
      {
         if(Math.random() < 0.95)
         {
            return "room";
         }
         return "tunnel";
      }
      if(this.nodeClass == "support")
      {
         if(Math.random() < this.__roomProbability)
         {
            return "room";
         }
         return "tunnel";
      }
   }
   function toString()
   {
      return "Node : " + this.x + " , " + this.y + ", type: " + this.nodeType + " , leftx: " + this.leftX + (this.nodeType != "room" ? "" : ", ( w: " + this.roomWidth + " , h: " + this.roomHeight + " a: " + this.roomAnchor + " )");
   }
}
